function results=maxent_bi(x,y,xrange,nx,yrange,ny,k,lam0)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% maxent code for bi-variate density
% x, y: input data
% xrange, yrange: range of x and y
% nx, ny: number of grid points on xrange and y range
% k: highest order of moments used in the estimation
% lam0: initial values for Lagrangian multipliers, default is a vector of zeros
%
% Author: Ximng Wu, Texas A&M University, email: xwu@tamu.edu
% This version: Dec. 28, 2009
% For details, See Wu, X. "Exponential Series Estimator of Multivariate Densities", Journal of Econometrics, forthcoming.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



incx=range(xrange)/(nx-1);
incy=range(yrange)/(ny-1);

X=xrange(1):incx:xrange(2);
Y=yrange(1):incy:yrange(2);
X=X';
Y=Y';

[xbase ybase]=meshgrid(X,Y);
xbase=reshape(xbase,[],1);
ybase=reshape(ybase,[],1);

mm=[];
Z=[];

for i=0:k
	for j=0:k-i
		mm=[mm x.^i.*y.^j];
		Z=[Z xbase.^i.*ybase.^j];
	end
end

mm=mean(mm);		
		


KK=length(mm);


lam=zeros(KK,1);
if nargin>7&~isempty(lam0)
        lam=lam0;
end


H=zeros(KK,KK);

change=1;

t=0;
inc=incx*incy;
while change>1e-9
	t=t+1;
	den=exp(-Z*lam)*inc;		

   for i=1:KK
   		Z2=Z(:,i).*den;
		for j=i:KK
			H(i,j)=Z2'*Z(:,j);
			H(j,i)=H(i,j);
		end
	end
	
							
	dev1=mm'-H(:,1);
	change=inv(H)*dev1;		
	lam=lam-change;
	change=dev1'*change/KK;
end

results.lam=lam;
results.hessian=H;
results.xbase=xbase;
results.ybase=ybase;
results.den=den/inc;
results.incx=incx;
results.incy=incy;
results.moment=mm;
results.ent=-mm*lam;
results.n=length(x);
results.it=t;